<!--
SPDX-FileCopyrightText: 2022-2023 nervuri <https://nervuri.net/contact>

SPDX-License-Identifier: BSD-3-Clause
-->

# TLS Client Hello Mirror

This test:

* reflects the complete Client Hello message, preserving the order in which TLS parameters and extensions are sent;
* can be used to check for TLS privacy pitfalls ([session resumption](https://svs.informatik.uni-hamburg.de/publications/2018/2018-12-06-Sy-ACSAC-Tracking_Users_across_the_Web_via_TLS_Session_Resumption.pdf), [TLS fingerprinting](https://tlsfingerprint.io/), [system time exposure](https://datatracker.ietf.org/doc/html/draft-mathewson-no-gmtunixtime));
* supports both HTTP and [Gemini](https://gemini.circumlunar.space/) on the same port;
* is [free as in freedom](https://www.gnu.org/philosophy/free-sw.en.html) and trivial to self-host.

## Installation

See [INSTALL.md](INSTALL.md).

## API documentation

This test exposes two JSON endpoints: [/json/v1 (basic)](https://tlsprivacy.nervuri.net/json/v1) and [/json/v2 (detailed)](https://tlsprivacy.nervuri.net/json/v2).  See [DOC.md](DOC.md) for details.

## Roadmap

* HTML & gemtext front-end
* detect client vulnerability to session [prolongation attacks](https://svs.informatik.uni-hamburg.de/publications/2018/2018-12-06-Sy-ACSAC-Tracking_Users_across_the_Web_via_TLS_Session_Resumption.pdf#page=3)
* support early data / 0-RTT (Go's `crypto/tls` library currently does not)
* support sessionID-based resumption (Go's `crypto/tls` library currently does not)

## Contributing

This project is hosted [at tildegit.org](https://tildegit.org/nervuri/client-hello-mirror).  If you don't want to make an account, just [shoot me an email](https://nervuri.net/contact) with your [patch](https://git-send-email.io/) / suggestion / bug report / whatever else.
